//
// sharpflash
// $Id: sharpflash.c 227 2007-10-19 21:49:22Z  $
//
// This program is used to read and write NAND flash
// on the Reciva Radio Module, via a JTAG Wiggler 
// interface.
//
// This program is very slow, as it uses the boundary
// scan path to directly drive the nand flash.  What it
// needs to do really, is use mem-read and mem-write, and
// make use of the ARM NAND Flash Controller.
//
// Copyright (C) 2007 Steve Clarke, Sharpfin Project www.sharpfin.zevv.nl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "def.h"
#include "jtag.h"
#include "ppt.h"
#include "pin2410.h"
#include "k9f2808.h"

void OpenPpt(void);

int main(int argc,char *argv[])
{
	U32 id32 ;
	U16 dev ;
	U32 start, len ;

	if (argc!=5 && argc!=2) {
		printf("\nsharpflash -r|-w|-b [ file.bin start len ]\n\n") ;
		printf("  -r -w -b - read, write or bad-block check  NAND Flash\n") ;
		printf("  file.bin - destination / source filename, where file.bin\n") ;
		printf("             is in nanddump format\n") ;
		printf("  start    - hex start position in NAND for read/write, where\n") ;
		printf("	     start must be a multiple of 4000(hex)\n") ;
		printf("  len      - hex length to read/write. if file is too short,\n") ;
		printf("	     NAND will be filled with 0xFF. len must be a\n") ; 
		printf("	     multiple of 4000(hex)\n\n") ;
		printf(" Output data contains:\n\n") ;
		printf("   w - page written OK\n") ;
		printf("   r - page read OK\n") ;
		printf("   . - page check OK\n") ;
		printf("   b - page/block identified as bad\n") ;
		printf("   B - page/block has just been marked bad\n\n") ;
		printf("v%s, http://www.sharpfin.zevv.nl/\n\n", VER) ;
		return 0;
	}

	printf("\nSharpfin Flash Programmer - http://www.sharpfin.zevv.nl/\n\n") ;

	// Open Parallel Port
	validPpt = GetValidPpt();
	if(!validPpt) {
		printf("Unable to find a parallel port\n");
		return 0 ;
	}
	SetPptCompMode();

	// Connect to the Processor
	id32=JTAG_ReadId();
	if (id32!=0x0032409d) {
		printf("Unable to find S3C2410 processor on JTAG Cable, found %X\n", id32) ;
		return 0 ;
	} else {
		printf("Found S3C2410 processor on JTAG Cable\n") ;
	}

	S2410_InitCell() ;

	K9F2808_JtagInit();
	K9F2808_init();

	// Check the NAND Flash
	dev=K9F2808_checkid();
	if (dev!=0xEC75) {
		printf("Unable to find K9F2808UOC on processor, found 0x%04X\n", dev) ;
		return 0 ;
	} else {
		printf("Found K9F2808UOC on processor\n") ;
	}


	if (strcmp(argv[1],"-r")==0 || strcmp(argv[1],"-w")==0) {
	
		if (argc!=5) {
			printf("sharpflash: start and/or length not supplied\n") ;
			return 0 ;
		}

		sscanf(argv[3], "%X", &start) ;
		sscanf(argv[4], "%X", &len) ;
	
		// Check the passed parameters
	
		if ( ((start/0x4000)*0x4000) != start ) {
			printf("start address must be a multiple of 4000(hex)\n") ;
			return 0 ;
		}

		if ( ((len/0x4000)*0x4000) != len ) {
			printf("len must be a multiple of 4000(hex)\n") ;
			return 0 ;
		}

		if (strcmp(argv[1],"-r")==0) {
	
			// Read data
			K9F2808_read(argv[2], start, len) ;

		} else if (strcmp(argv[1],"-w")==0) {
			// Write data
			K9F2808_write(argv[2], start, len) ;
		}

	} else if (strcmp(argv[1],"-b")==0) {

		// Check Bad Blocks
		K9F2808_badcheck() ;


	} else {

		printf("Unknown command: %s\n", argv[1]) ;

	}

	printf("\n") ;
	return 0 ;

}

